require 'json'
require 'socket.io-client-simple'
require 'eventmachine'

module EventMachine
  # Communicates with MeshBlu(tm) via websocket connection.
  class MeshRuby
    attr_accessor :socket, :uuid, :token, :queue, :url

    def initialize(uuid, token, url = 'wss://meshblu.octoblu.com:443')
      @uuid, @token, @url, @queue = uuid, token, url, EventMachine::Queue.new
      @socket    = SocketIO::Client::Simple::Client.new url
    end

    def connect
      socket.connect
      create_socket_events
      self
    end

    ### Bootstraps all the events for MeshBlu in the correct order.
    def create_socket_events
      #OTHER EVENTS: :identify, :identity, :ready, :disconnect, :message
      this = self; socket = this.socket # socket.io-client bugs?
      socket.on :connect do
        socket.on :identify do |data|
          auth = {uuid: this.uuid, token: this.token}
          emit :identity, auth
        end
        socket.on(:message) { |msg| this.push(msg); this.pop }
      end
    end

    # Sanitize messages, making a best effort attempt to hashify them, otherwise
    # returns message as-is.
    def push(message)
      @queue.push message.is_a?(String) ? JSON.parse(message) : message
    rescue JSON::ParserError
      @queue.push message
    end

    def pop
      @queue.pop(&@onmessage) if @onmessage
    end

    # TODO: Rename to 'message', since this is not the only thing we're emitting
    def emit(devices, message_hash)
      socket.emit("message", message_hash.merge(devices: devices))
    end

    def credentials
      @credentials ||= {uuid: @uuid, token: @token}
    end

    def data(telemetry)
      case telemetry
      when Hash then nil # do stuff
      else telemetry = {data: telemetry}
      end
      socket.emit("data", telemetry.merge!(credentials))
    end

    def onmessage(&blk)
      @onmessage = blk
    end

    def toggle_debug!
      if @debugger
        socket.remove_listener(@debugger)
      else
        @debugger = socket.on(:*) { |a, b| puts "#{a} #{b}" }
      end
    end
  end
end
